"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const os_1 = tslib_1.__importDefault(require("os"));
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const models_cloud_1 = tslib_1.__importDefault(require("@tradle/models-cloud"));
const errors_1 = require("./errors");
const utils_1 = require("./utils");
const regions = models_cloud_1.default['tradle.cloud.AWSRegion'].enum.map(({ id, title }) => ({
    name: title,
    value: id.replace(/[.]/g, '-')
}));
const PROFILE_REGEX = /^\[(?:[^\s]*?\s)?(.*)\]$/;
const getProfileName = line => line.match(PROFILE_REGEX)[1];
const parseConf = conf => ({
    profiles: conf.split('\n')
        .filter(s => s.startsWith('['))
        .map(getProfileName)
});
const getProfiles = () => {
    let conf;
    try {
        conf = fs_1.default.readFileSync(`${os_1.default.homedir()}/.aws/credentials`, { encoding: 'utf8' });
    }
    catch (err) {
        return ['default'];
    }
    const { profiles } = parseConf(conf);
    return profiles.includes('default') ? profiles : ['default'].concat(profiles);
};
exports.init = (conf) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const defaultWhen = answers => answers.overwriteEnv !== false;
    const chooseFlow = [
        {
            type: 'confirm',
            name: 'overwriteEnv',
            message: 'This will overwrite your .env file',
            when: () => fs_1.default.existsSync('./.env')
        },
        {
            type: 'confirm',
            name: 'haveRemote',
            message: 'Have you already deployed your MyCloud to AWS?'
        }
    ];
    const getLocal = [
        {
            type: 'confirm',
            name: 'haveLocal',
            message: 'Do you have a local development environment? (a clone of https://github.com/tradle/serverless)'
        },
        {
            type: 'input',
            name: 'projectPath',
            message: 'Enter the path to your local development environment (a clone of https://github.com/tradle/serverless)',
            when: answers => defaultWhen(answers) && answers.haveLocal,
            validate: local => {
                if (!utils_1.isValidProjectPath(local)) {
                    return 'Provided path doesn\'t contain a serverless.yml, please try again';
                }
                return true;
            }
        }
    ];
    const flow = yield inquirer_1.default.prompt(chooseFlow);
    if (!flow.haveRemote) {
        return Object.assign({}, flow, (yield inquirer_1.default.prompt(getLocal)));
    }
    const getRemoteAndLocal = [
        {
            type: 'list',
            name: 'region',
            message: 'Which AWS region is your deployment in?',
            choices: regions
        },
        {
            type: 'list',
            name: 'awsProfile',
            message: 'Select your aws profile',
            when: defaultWhen,
            choices: getProfiles()
                .map(profile => ({
                name: profile,
                value: profile
            }))
                .concat([
                new inquirer_1.default.Separator(),
                {
                    name: 'Other (specify)',
                    value: null
                }
            ]),
        },
        {
            type: 'input',
            name: 'awsProfile',
            message: 'profile',
            when: answers => answers.overwriteEnv && !answers.awsProfile
        },
        {
            type: 'list',
            name: 'stack',
            message: 'Which Tradle stack will you be configuring?',
            when: defaultWhen,
            choices: ({ region, awsProfile }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const stackInfos = yield conf.getStacks({
                    profile: awsProfile,
                    region
                });
                if (!stackInfos.length) {
                    throw new Error('no stacks found');
                }
                return stackInfos.map(({ name, id }) => ({
                    name,
                    value: { name, id }
                }));
            })
        },
    ]
        .concat(getLocal)
        .concat({
        type: 'confirm',
        name: 'loadCurrentConf',
        message: 'Would you like to pull your current configuration?',
    });
    return Object.assign({}, flow, (yield inquirer_1.default.prompt(getRemoteAndLocal)));
});
exports.fn = (conf, message) => {
    return inquirer_1.default.prompt([
        {
            type: 'list',
            name: 'fn',
            message,
            choices: conf.getFunctions
        }
    ])
        .then(({ fn }) => fn);
};
exports.confirm = (message, defaultValue = true) => {
    return inquirer_1.default.prompt([
        {
            type: 'confirm',
            name: 'confirm',
            message,
            default: defaultValue,
        }
    ])
        .then(({ confirm }) => confirm);
};
exports.confirmOrAbort = (msg) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const confirmed = yield exports.confirm(msg);
    if (!confirmed) {
        throw new errors_1.Errors.UserAborted();
    }
});
exports.ask = (message) => {
    return inquirer_1.default.prompt([
        {
            type: 'input',
            name: 'answer',
            message
        }
    ])
        .then(({ answer }) => answer);
};
const getEC2KeyPair = (client) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    let key;
    const originalMessage = 'What is the name of the EC2 key pair you configured in AWS?';
    let message = originalMessage;
    while (!key) {
        key = yield exports.ask(message);
        if (!(yield utils_1.doKeyPairsExist(client, [key]))) {
            key = null;
            message = `Key pair not found. ${originalMessage}`;
        }
    }
    return key;
});
exports.chooseEC2KeyPair = (client) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const know = yield exports.confirm('Do you know the name of the EC2 key pair you want to use?');
    if (know) {
        return getEC2KeyPair(client);
    }
    return exports.choose({
        message: 'Choose the key pair to set up for SSH access',
        choices: () => utils_1.listKeyPairs(client),
    });
});
exports.choose = ({ message, choices }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    return inquirer_1.default.prompt([
        {
            type: 'list',
            name: 'choice',
            message,
            choices: typeof choices === 'function' ? choices : () => choices,
        }
    ])
        .then(({ choice }) => choice);
});
exports.chooseRegion = (opts) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const message = opts && opts.message || 'Choose a deployment region';
    return exports.choose({
        message,
        choices: regions,
    });
});
exports.chooseAZs = (client, { region, count }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const azs = yield utils_1.listAZs(client, { region });
    if (azs.length === count)
        return azs;
    const chosen = [];
    let choice;
    let message;
    while (chosen.length < count) {
        message = `Choose an availability zone (${(chosen.length + 1)} of ${count})`;
        choice = yield exports.choose({ message, choices: azs });
        azs.splice(azs.indexOf(choice), 1);
        chosen.push(choice);
    }
    return chosen;
});
//# sourceMappingURL=prompts.js.map