"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const crypto_1 = tslib_1.__importDefault(require("crypto"));
const path_1 = tslib_1.__importDefault(require("path"));
const fs_1 = tslib_1.__importDefault(require("fs"));
const lodash_1 = tslib_1.__importDefault(require("lodash"));
const shelljs_1 = tslib_1.__importDefault(require("shelljs"));
const node_fetch_1 = tslib_1.__importDefault(require("node-fetch"));
const models_pack_1 = tslib_1.__importDefault(require("@tradle/models-pack"));
const validate_resource_1 = tslib_1.__importDefault(require("@tradle/validate-resource"));
const errors_1 = tslib_1.__importDefault(require("@tradle/errors"));
const empty_bucket_1 = require("./empty-bucket");
const models_1 = require("./models");
const errors_2 = require("./errors");
exports.get = (url) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const res = yield node_fetch_1.default(url);
    if (res.statusCode > 300) {
        throw new Error(res.statusText);
    }
    return yield res.json();
});
exports.getNamespace = ({ models, lenses }) => {
    const model = lodash_1.default.values(models)[0] || lodash_1.default.values(lenses)[0];
    return model && models_pack_1.default.getNamespace(model.id);
};
exports.pack = ({ namespace, models, lenses }) => {
    if (!(lodash_1.default.size(models) || lodash_1.default.size(lenses))) {
        throw new errors_2.Errors.InvalidInput('expected "models" and/or "lenses"');
    }
    if (!namespace) {
        namespace = exports.getNamespace({ models, lenses });
    }
    const modelsPack = models_pack_1.default.pack({ namespace, models, lenses });
    return modelsPack;
};
exports.validateResource = resource => validate_resource_1.default({
    models: models_1.models,
    resource
});
exports.prettify = obj => {
    return typeof obj === 'string' ? obj : JSON.stringify(obj, null, 2);
};
exports.isValidProjectPath = project => {
    return fs_1.default.existsSync(path_1.default.resolve(project, 'serverless.yml'));
};
exports.toEnvFile = obj => Object.keys(obj)
    .map(key => `${key}="${obj[key]}"`)
    .join('\n');
const acceptAll = (item) => true;
exports.listStackResources = (aws, StackName, filter = acceptAll) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    let resources = [];
    const opts = { StackName };
    while (true) {
        let { StackResourceSummaries, NextToken } = yield aws.cloudformation.listStackResources(opts).promise();
        resources = resources.concat(StackResourceSummaries.filter(filter));
        opts.NextToken = NextToken;
        if (!opts.NextToken)
            break;
    }
    return resources;
});
exports.listStackResourcesByType = (type, aws, stackName) => {
    return exports.listStackResources(aws, stackName, ({ ResourceType }) => type === ResourceType);
};
exports.listStackResourceIdsByType = (type, aws, stackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const resources = yield exports.listStackResourcesByType(type, aws, stackName);
    return resources.map(r => r.PhysicalResourceId);
});
exports.listStackBuckets = exports.listStackResourcesByType.bind(null, 'AWS::S3::Bucket');
exports.listStackBucketIds = exports.listStackResourceIdsByType.bind(null, 'AWS::S3::Bucket');
exports.listStackFunctions = exports.listStackResourcesByType.bind(null, 'AWS::Lambda::Function');
exports.listStackFunctionIds = exports.listStackResourceIdsByType.bind(null, 'AWS::Lambda::Function');
exports.destroyBucket = (aws, Bucket) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    yield empty_bucket_1.emptyBucket(aws.s3, Bucket);
    try {
        yield aws.s3.deleteBucket({ Bucket }).promise();
    }
    catch (err) {
        errors_1.default.ignore(err, [
            { code: 'ResourceNotFoundException' },
            { code: 'NoSuchBucket' },
        ]);
    }
});
exports.disableStackTerminationProtection = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    return yield aws.cloudformation.updateTerminationProtection({
        StackName,
        EnableTerminationProtection: false,
    }).promise();
});
exports.deleteStack = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    while (true) {
        try {
            yield aws.cloudformation.deleteStack({ StackName }).promise();
            return () => exports.awaitStackDelete(aws, StackName);
        }
        catch (err) {
            if (!err.message.includes('UPDATE_ROLLBACK_COMPLETE_CLEANUP_IN_PROGRESS')) {
                throw err;
            }
            yield exports.wait(5000);
        }
    }
});
exports.createStack = (aws, params) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    yield aws.cloudformation.createStack(params).promise();
    return () => exports.awaitStackCreate(aws, params.StackName);
});
exports.updateStack = (aws, params) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    yield aws.cloudformation.updateStack(params).promise();
    return () => exports.awaitStackUpdate(aws, params.StackName);
});
const isUpdateableStatus = (status) => {
    return status.endsWith('_COMPLETE') && !status.startsWith('DELETE_');
};
exports.getStackId = (aws, stackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const stacks = yield exports.listStacks(aws, ({ StackName, StackStatus }) => StackName === stackName && isUpdateableStatus(StackStatus));
    return stacks[0] && stacks[0].id;
});
exports.awaitStackCreate = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    return yield aws.cloudformation.waitFor('stackCreateComplete', { StackName }).promise();
});
exports.awaitStackDelete = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    return yield aws.cloudformation.waitFor('stackDeleteComplete', { StackName }).promise();
});
exports.awaitStackUpdate = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    return yield aws.cloudformation.waitFor('stackUpdateComplete', { StackName }).promise();
});
exports.listStacks = (aws, filter = acceptAll) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const listStacksOpts = {
        StackStatusFilter: ['CREATE_COMPLETE', 'UPDATE_COMPLETE', 'UPDATE_ROLLBACK_COMPLETE']
    };
    let stackInfos = [];
    let keepGoing = true;
    while (keepGoing) {
        let { NextToken, StackSummaries } = yield aws.cloudformation.listStacks(listStacksOpts).promise();
        stackInfos = stackInfos.concat(StackSummaries.filter(filter).map(({ StackId, StackName }) => ({
            id: StackId,
            name: StackName
        })));
        listStacksOpts.NextToken = NextToken;
        keepGoing = !!NextToken;
    }
    return stackInfos;
});
exports.getApiBaseUrl = (aws, StackName) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const result = yield aws.cloudformation.describeStacks({ StackName }).promise();
    const { Outputs } = result.Stacks[0];
    const endpoint = Outputs.find(x => /^ServiceEndpoint/.test(x.OutputKey));
    return endpoint.OutputValue;
});
exports.splitCamelCase = str => str.split(/(?=[A-Z])/g);
exports.checkCommandInPath = cmd => {
    const { code } = shelljs_1.default.exec(`command -v ${cmd}`);
    if (code !== 0) {
        throw new errors_2.Errors.InvalidEnvironment(`Please install: ${cmd}`);
    }
};
exports.wait = millis => new Promise(resolve => setTimeout(resolve, millis));
exports.normalizeNodeFlags = flags => {
    if (!(flags.inspect || flags['inspect-brk'] || flags.debug || flags['debug-brk']))
        return;
    const nodeVersion = Number(process.version.slice(1, 2));
    if (nodeVersion >= 8) {
        if (flags.debug) {
            delete flags.debug;
            flags.inspect = true;
        }
        if (flags['debug-brk']) {
            delete flags['debug-brk'];
            flags['inspect-brk'] = true;
        }
    }
    else {
        if (flags.debug || flags['debug-brk']) {
            flags.inspect = true;
        }
    }
};
exports.pickNonNull = obj => lodash_1.default.pickBy(obj, val => val != null);
const sha256 = (str, enc = 'hex') => {
    return crypto_1.default.createHash('sha256').update(str).digest(enc);
};
const shortenString = (str, maxLength) => {
    const tooLongBy = str.length - maxLength;
    if (tooLongBy <= 0)
        return str;
    if (str.length < 6)
        throw new Error(`string is too short: ${str}`);
    return str.slice(0, maxLength - 6) + sha256(str).slice(0, 6);
};
exports.getServicesStackName = (stackName) => {
    const name = stackName.match(/^tdl-(.*?)-ltd-[a-zA-Z]+$/)[1];
    const shortName = shortenString(name, 14);
    return `${shortName}-srvcs`;
};
exports.canAccessECRRepos = (aws, { accountId, repoNames }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    try {
        yield aws.ecr.describeRepositories({
            registryId: accountId,
            repositoryNames: repoNames
        }).promise();
        return true;
    }
    catch (err) {
        if (err.name === 'AccessDeniedException') {
            return false;
        }
        throw err;
    }
});
exports.doKeyPairsExist = (aws, names) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const params = {
        KeyNames: names
    };
    try {
        yield aws.ec2.describeKeyPairs(params).promise();
        return true;
    }
    catch (err) {
        return false;
    }
});
exports.listKeyPairs = (aws) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { KeyPairs } = yield aws.ec2.describeKeyPairs().promise();
    return KeyPairs.map(k => k.KeyName);
});
exports.listAZs = (aws, { region }) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { AvailabilityZones } = yield aws.ec2.describeAvailabilityZones().promise();
    return AvailabilityZones.filter(a => a.RegionName === region);
});
exports.getUsedEIPCount = (aws) => tslib_1.__awaiter(this, void 0, void 0, function* () {
    const { ElasticIps } = yield aws.opsworks.describeElasticIps().promise();
    return ElasticIps.length;
});
//# sourceMappingURL=utils.js.map